
//  @(#)option.h 1.7 94/02/03
//
//  Contains definitions for class "Option".
//
//  Copyright (c) Steve Holden and Rich Burridge.
//                All rights reserved.
//
//  Permission is given to distribute these sources, as long as the
//  copyright messages are not removed, and no monies are exchanged.
//
//  No responsibility is taken for any errors or inaccuracies inherent
//  either to the comments or the code of this program, but if
//  reported to me then an attempt will be made to fix them.

#include <X11/Xlib.h>
#include <X11/Xresource.h>

#define  FCLOSE   (void) fclose
#define  FPRINTF  (void) fprintf
#define  FPUTS    (void) fputs
#define  FREE     (void) free
#define  PCLOSE   (void) pclose
#define  PUTC     (void) putc
#define  SSCANF   (void) sscanf
#define  SPRINTF  (void) sprintf
#define  STRCAT   (void) strcat
#define  STRCPY   (void) strcpy
#define  STRNCPY  (void) strncpy
#define  UNGETC   (void) ungetc

#define  DEFCOLUMNS    1      // Number of columns per page.
#define  DEFCOPIES     1      // Number of copies of each file.
#define  DEFMARGIN     8      // Default margin size (in points).
#define  LINELENGTH    80     // Number of characters per line.
#define  MAXCOPIES     500    // Maximum number of copies to print.
#define  MAXHDRS       64     // Maximum number of -addhdr or -remhdr options.
#define  MAXLINE       256    // Maximum string length.
#define  MAXMARGIN     500    // Maximum margin size (in points).
#define  MAXNAMES      512    // Maximum number of files from command line.
#define  MAXRESOURCES  36     // Maximum number of mp X resources.
#define  NAMEFIELDS    3      // Default no. of "words" from passwd gecos.
#define  NAMELENGTH    40     // Maximum allowable real user name.
#define  PAGELENGTH    72     // Number of lines per page.
#define  TABSIZE       8      // Number of spaces in a tab char.

#define  PRINTPROG    "/usr/spool/lp"   // BSD or System V system?

#define  ARG_EQUAL(s, n)      (!argcmp(argv[i], s, n))
#define  CASE_EQUAL(a, b)     (!strcasecmp(a, b))
#define  EQUAL(a, b)          (!strcmp(a, b))
#define  OLD_ARG_EQ(s)        (!strcmp(argv[i], s))

#ifndef  MAXPATHLEN
#define  MAXPATHLEN   1024
#endif // MAXPATHLEN

#ifndef  PROLOGUE                             // PostScript prologue file.
#define  PROLOGUE          "/usr/local/lib/mp"
#endif // PROLOGUE

#define  PRO_FILOFAX       "mp.pro.ff.ps"
#define  PRO_FRANKLIN      "mp.pro.fp.ps"
#define  PRO_LANDSCAPE     "mp.pro.l.ps"
#define  PRO_NORMAL        "mp.pro.ps"
#define  PRO_OTHER         "other"
#define  PRO_PRO_PLAN      "mp.pro.pp.ps"
#define  PRO_TIME_MANAGER  "mp.pro.tm.ps"
#define  PRO_TS_INTL       "mp.pro.tsi.ps"
#define  PRO_TS_PARTNER    "mp.pro.tsp.ps"

enum paper_type  { P_NONE, P_A4, P_US } ;

enum res_type   { R_FTYPE,    R_OTYPE,    R_ORGANISER, R_PSIZE,
                  R_HDRPAGE,  R_MAIL_FDR, R_MAIL_DIG,  R_SUBJECT,
                  R_LLENGTH,  R_PLENGTH,  R_COPIES,    R_PNAME,
                  R_DSIDED,   R_ADDHDRS,  R_REMHDRS,   R_PROLOG,
                  R_ALIAS,    R_NCOLUMNS, R_TABSIZE,   R_NAMEC,
                  R_NAMEW,    R_WRAP,     R_FLIP,      R_FORCEPAGE,
                  R_ELM,      R_NUMBER,   R_FROM,      R_CONTENT,
                  R_MOD,      R_TUMBLE,   R_ALLHDRS,   R_BSDPRINT,
                  R_BMARGIN,  R_LMARGIN,  R_RMARGIN,   R_TMARGIN } ;

class Option {
  public:

  char *addhdrs[MAXHDRS] ;       // List of additional headers to be printed.
  char *aliasname ;              // User name alias on output.
  char *appname ;                // Application (program) name.
  char *configfile ;             // Alternate X resource file.
  char *gsubject ;               // Global subject set from command line.
  char *namelist[MAXNAMES] ;     // List of file names from argv.
  char *printer_name ;           // Printer name (if any) for output.
  char *prologue ;               // Name of PostScript prologue file.
  char proname[MAXPATHLEN] ;     // Full pathname of the prologue file.
  char *ptype ;                  // Prologue type.
  char *remhdrs[MAXHDRS] ;       // List of headers not to be printed.

  int addhdr_num ;           // Number of additional header lines to print.
  int allhdrs ;              // If set, print out all mail/news headers.
  int article ;              // Set for news in "Article from " format.
  int banner ;               // Print a burst page on the printer?
  int bsdprint ;             // Set if this is a BSD printer system.
  int content ;              // Set if Content-Length: has message length.
  int copies ;               // Number of copies of each file to print.
  int cpos ;                 // Current column position for usage output.
  int digest ;               // Are we are printing a mail digest.
  int duplex ;               // Print on both sides of the paper?
  int elm_if ;               // ELM mail frontend intermediate file format.
  int flip ;                 // Flip top and bottom banners.
  int folder ;               // Set if we are printing a mail folder.
  int forcepage ;            // Force next page to a new page or column?
  int landscape ;            // Set if we are printing in landscape mode.
  int llen ;                 // Number of characters per line.
  int modtime ;              // If set, use last modification time.
  int namefields ;           // Number of "words" from passwd gecos.
  int namelength ;           // Maximum # of chars. from passwd gecos.
  int number ;               // If set, than number each output line.
  int numcols ;              // Number of columns per page.
  int numnames ;             // Number of file names from command line.
  int plen ;                 // Number of lines per page.
  int print_orig ;           // Print From rather than To in mail header.
  int print_ps ;             // Print PostScript files if set.
  int remhdr_num ;           // Number of additional header lines not to print.
  int tabsize ;              // Number of spaces in a tab character.
  int text_doc ;             // Printing normal text.
  int toprinter ;            // Send output to the printer?
  int tumble ;               // Tumble the duplex'ed page?
  int wrapwords ;            // Wrap words of output.

  int bmargin ;              // Bottom margin size (in points).
  int lmargin ;              // Left margin size   (in points).
  int rmargin ;              // Right margin size  (in points).
  int tmargin ;              // Top margin size    (in points).

  int cols_given ;           // "-columns    #" command line option given?
  int llen_given ;           // "-linelength #" command line option given?
  int plen_given ;           // "-pagelength #" command line option given?

  paper_type paper_size ;    // Paper size - default None.

       Option                 () ;

  void adjust_paper_size      (void) ;
  void get_appname            (int, char **) ;
  void get_hdr_arg            (char *, char **, int *) ;
  void get_options            (int, char **) ;
  void read_resources         (void) ;
  void write_resources        (void) ;

  private:

  char *mpres[MAXRESOURCES] ;          // Mp resource names.
  XrmDatabase rDB ;                    // Mp X resources database.

  char *bool                  (int) ;
  char *get_resource          (enum res_type) ;

  int  argcmp                 (char *, char *, int) ;
  int get_bool_resource       (enum res_type, int *) ;
  int get_int_resource        (enum res_type, int *) ;
  int get_str_resource        (enum res_type, char *) ;
  int save_resources          (void) ;

  void get_hdr_resource       (enum res_type, char *) ;
  void load_mp_defs           (char *) ;
  void put_hdr_resource       (enum res_type) ;
  void put_resource           (enum res_type, char *) ;
  void read_str               (char **, char *) ;
  void usage                  (void) ;
  void usageoption            (char *) ;
} ;
