
//  @(#)input.C 1.3 93/11/18
//
//  Copyright (c) Steve Holden and Rich Burridge.
//                All rights reserved.
//
//  Permission is given to distribute these sources, as long as the
//  copyright messages are not removed, and no monies are exchanged.
//
//  No responsibility is taken for any errors inherent either
//  to the comments or the code of this program, but if reported
//  to me then an attempt will be made to fix them.

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <pwd.h>
#include "option.h"
#include "header.h"
#include "input.h"
#include "print.h"

extern Header hdr ; 
extern Input  input ; 
extern Option opt ;
extern Print  prt ; 


void Input::
init_setup(void)        // Set default values for various options.
{
  char *c ;
  int amp_cnt = 0 ;     // Number of ampersands in gecos field.
  int i, len ;
  struct passwd *pp ;

  c = getlogin() ;                 // Pointer to users login name.
  if (c == NULL || *c == '\0')     // Get username from password file.
    {
      pp = getpwuid(geteuid()) ;
      if (pp == NULL) c = "printing" ;
      else c = pp->pw_name ;
    }
  if (opt.aliasname && *opt.aliasname != '\0')
    c = opt.aliasname ;            // Use alias if defined.
  owner = (char *) malloc((unsigned) (strlen(c) + 1)) ;
  STRCPY(owner, c) ;
  whoami = (char *) malloc((unsigned) (strlen(c) + 1)) ;
  STRCPY(whoami, c) ;              // Save User login name.

//  Have a look for the users gecos (normally real name), so that its a bit
//  more recognisable. If this field is too long, then we need to truncate
//  sensibly. We also need to check a few things. If we've extracted
//  namefields "words" or have found a comma, then exit. If an ampersand is
//  found, this is expanded to the users name in capitals.

  pp = getpwnam(owner) ;
  if (pp != NULL && pp->pw_gecos && pp->pw_gecos[0] != '\0')
    {
      len = strlen(pp->pw_gecos) ;
      for (i = 0; i < len; i++)
        if (pp->pw_gecos[i] == '&') amp_cnt++ ;

      nameptr = getenv("NAME") ;
      if (nameptr != NULL) len = strlen(nameptr) ;
      else len = strlen(pp->pw_gecos) + amp_cnt * strlen(c) ;

      owner = (char *) malloc((unsigned) (len + 1)) ;

      if (nameptr != NULL)
        process_name_field(c, nameptr, opt.namefields, opt.namelength) ;
      else
        process_name_field(c, pp->pw_gecos, opt.namefields, opt.namelength) ;
    }

  if (opt.text_doc) doc_type = DO_TEXT ;
  switch (doc_type)
    {
      case DO_TEXT : prt.message_for = "Listing for ";
                     opt.digest = 0 ;
                     break ;
      case DO_MAIL : prt.message_for = opt.digest ? "Mail digest for "
                                                  : "Mail for " ;
                     break ;
      case DO_NEWS : prt.message_for = opt.digest ? "News digest for "
                                                  : "News for " ;
                     break ;
    }
}


Input::
Input()
{
  int i ;

  for (i = 0; i < MAXHDRS; i++) hdrs[i] = NULL ;

  end_of_file = 0 ;          // EOF indicator.
  linenum     = 0 ;          // Current line number from input file.
  overflow    = 0 ;          // Initially no text wraparound.
  doc_type    = DO_MAIL ;    // Printing type - default mail.
}


// Extract user name from $NAME or passwd GECOS.
 
void Input::
process_name_field(char *c, char *ptr, int fields, int length)
{
  int i, j, len, n, spaces, slen ;

  n = spaces = 0 ;
  slen = strlen(ptr) ;
  for (i = 0; i < slen; i++)
    {
           if (*ptr == ',') break ;
      else if (*ptr == '&')
        {
          if (islower(c[0])) owner[n++] = toupper(c[0]) ;
          len = strlen(c) ;
          for (j = 1; j < len; j++) owner[n++] = c[j] ;
          ptr++ ;
        }
      else if (*ptr == ' ' || *ptr == '\t')
        {
          if (++spaces == fields) break ;
          else
            while (*ptr == ' ' || *ptr == '\t') owner[n++] = *ptr++ ;
        }
      else owner[n++] = *ptr++ ;
      if (n >= length) break ;
    }
  if (n > length) n = length ;
  owner[n] = '\0' ;
}


//  Read an input line into nextline, setting end_of_file, end_of_page
//  and end_of_line appropriately.

void Input::
readline(void)
{
  char *ptr ;
  int c ;
  int i = 0 ;         // Index into current line being read.
  int len = 0 ;       // Length of the current line.
  int startpos = 0 ;  // Start position of real text in line.

  if (end_of_file) return ;
  end_of_page = end_of_line = 0 ;

  if (opt.number)     // Should we number output lines?
    {
      SPRINTF(nextline, "%6d  ", ++linenum) ;
      startpos = len = i = 8 ;
    }

  if (overflow)       // Is this -wrap with word wraparound?
    {
      STRNCPY(&nextline[len], iobuf, overflow) ;
      len += overflow ;
      i += overflow ;
      overflow = 0 ;
    }

  while (len < opt.llen && (c = getc(ifp)) != EOF && c != '\n' && c != '\f')
    {
      if (c == '\t')
        {
          do
            {
              nextline[i++] = ' ' ;
              len++ ;
            }
          while (len % opt.tabsize != 0 && len <= opt.llen) ;
        }
      else
        { 
          nextline[i++] = c ;
          len++ ;
        }
      if (c == '\b')
        {
          len -= 2 ;
          i -= 2 ;
        }
    }
  nextline[i] = '\0' ;

  if (len == opt.llen && c != EOF && c != '\n' && c != '\f')
    {
      c = getc(ifp) ;
      if (c != EOF && c != '\n' && c != '\f') UNGETC(c, ifp) ;

      if (opt.wrapwords && (ptr = strrchr(nextline, ' ')) != NULL)
        {
          STRCPY(iobuf, ptr+1) ;
          len           = ptr - nextline + 1 ;
          nextline[len] = '\0' ;                   // Terminate current line.
          overflow      = opt.llen - len ;         // # of chars wrapped.
        }
    }

  if (opt.elm_if && c == '\f')
    {
      len-- ;
      i-- ;
    }

  switch (c)
    {
      case EOF  : if (i == startpos) end_of_file = 1 ;
                  else
                    { 
                      UNGETC(c, ifp) ;
                      end_of_line = 1 ;
                    }
                  break ;
      case '\n' : end_of_line = 1 ;
                  break ;

//  /usr/ucb/mail for some unknown reason, appends a bogus formfeed at
//  the end of piped output. The next character is checked; if it's an
//  EOF, then end_of_file is set, else the character is put back.

      case '\f' : if ((c = getc(ifp)) == EOF) end_of_file = 1 ;
                  else UNGETC(c, ifp) ;

                  end_of_line = 1 ;
                  end_of_page = 1 ;
                  break ;
    }
  clen = len + 1 ;          // Current line length (includes newline).
}
