
//  @(#)base.C 1.3 94/02/03
//
//  Various routines associated with the input popup.
//
//  Copyright (c) Rich Burridge.    All rights reserved.
//
//  Permission is given to distribute these sources, as long as the
//  copyright messages are not removed, and no monies are exchanged.
//
//  No responsibility is taken for any errors or inaccuracies inherent
//  either to the comments or the code of this program, but if
//  reported to me then an attempt will be made to fix them.

#include "mptool.h"
#include "patchlevel.h"
#include "mptool.xbm"
#include "mptool.xpm"
#include "printing.xbm"
#include "printing.xpm"
#include <Xm/AtomMgr.h>
#include <Xm/FileSB.h>
#include <Xm/Text.h>


Base::
Base()
{
  filenames = NULL ;     // List of file(s) to print.
  fpopup    = NULL ;     // No file selection popup window initially.
  fwarr     = NULL ;
  warr      = NULL ;
}


void Base::
do_copies_dec(void)
{
  if (opt.copies > 1) opt.copies-- ;
  x.set_int_value(F_MAIN, WI_COPIES, opt.copies) ;
}


void Base::
do_copies_inc(void)
{
  if (opt.copies < MAXCOPIES) opt.copies++ ;
  x.set_int_value(F_MAIN, WI_COPIES, opt.copies) ;
}


void Base::
do_exit_mptool(void)
{
  exit(0) ;
}


void Base::
do_file_apply(void)
{
  char all[BUFFERSIZE], *previous, *newfile ;
  Widget textw ;

  previous = x.get_str_value(F_MAIN, WI_FILENAME) ;
  textw    = XmFileSelectionBoxGetChild(fwarr[WI_FILE_FRAME], XmDIALOG_TEXT) ;
  newfile  = XmTextGetString(textw) ;
  all[0] = '\0' ;
  if (previous && *previous) STRCAT(all, previous) ;
  if (newfile  && *newfile)
    {
      if (previous && *previous) STRCAT(all, " ") ;
      STRCAT(all, newfile) ;
    }
  x.set_str_value(F_MAIN, WI_FILENAME, all) ;
}


void Base::
do_file_dismiss(void)
{
  XtUnmapWidget(fpopup) ;
  XtUnmanageChild(fpopup) ;
}


void Base::
do_print_files(void)
{
  char *filenames = x.get_str_value(F_MAIN, WI_FILENAME) ;

  do_print_cmd(filenames) ;
}


void Base::
do_reset_options(void)
{
       if (opt.article)  x.set_excl_value(F_MAIN, WI_FILE_TYPE_NEWS) ;
  else if (opt.print_ps) x.set_excl_value(F_MAIN, WI_FILE_TYPE_POST) ;
  else if (opt.text_doc) x.set_excl_value(F_MAIN, WI_FILE_TYPE_TEXT) ;
 
  x.set_excl_value(F_MAIN, WI_OUTPUT_PORT) ;
       if (opt.landscape)
    x.set_excl_value(F_MAIN, WI_OUTPUT_LAND) ;
  else if (!EQUAL(opt.ptype, PRO_NORMAL))
    x.set_excl_value(F_MAIN, WI_OUTPUT_PERORG) ;
 
  x.set_excl_value(F_MAIN, WI_ORG_TYPE_FF) ;   // Default to Filofax.

       if (EQUAL(opt.ptype, PRO_FRANKLIN))
    x.set_excl_value(F_MAIN, WI_ORG_TYPE_FP) ;
  else if (EQUAL(opt.ptype, PRO_PRO_PLAN))
    x.set_excl_value(F_MAIN, WI_ORG_TYPE_PP) ;
  else if (EQUAL(opt.ptype, PRO_TIME_MANAGER))
    x.set_excl_value(F_MAIN, WI_ORG_TYPE_TM) ;
  else if (EQUAL(opt.ptype, PRO_TS_INTL))
    x.set_excl_value(F_MAIN, WI_ORG_TYPE_TSI) ;
  else if (EQUAL(opt.ptype, PRO_TS_PARTNER))
    x.set_excl_value(F_MAIN, WI_ORG_TYPE_TSP) ;

       if (opt.paper_size == P_A4) x.set_excl_value(F_MAIN, WI_PAPER_SIZE_A4) ;
  else if (opt.paper_size == P_US) x.set_excl_value(F_MAIN, WI_PAPER_SIZE_US) ;

  x.set_str_value(F_MAIN, WI_SUBJECT,  opt.gsubject) ;
  x.set_int_value(F_MAIN, WI_COPIES,   opt.copies) ;
  x.set_str_value(F_MAIN, WI_FILENAME, filenames) ;

  if (opt.duplex) x.set_excl_value(F_MAIN, WI_NO_SIDES_DOUBLE) ;
  else            x.set_excl_value(F_MAIN, WI_NO_SIDES_SINGLE) ;
}


void Base::
do_save_defaults(void)
{
  get_main_vals() ;
  opt.write_resources() ;
}


void Base::
do_show_base(void)
{
  char title[MAXLINE] ;

  (void) tu_main_frame_widget("mptool", NULL, &warr) ;
  COMPOUND_TEXT = XmInternAtom(XtDisplay(warr[WI_MAIN_FRAME]),
                               "COMPOUND_TEXT", False) ;
#ifndef NO_DND
  x.init_dnd(warr[WI_MAIN_FORM]) ;
#endif // NO_DND

  help.add_help(warr[WI_MAIN_FORM],          wi_main_frame_help) ;
  help.add_help(warr[WI_FILENAME_LABEL],     wi_filename_help) ;
  help.add_help(warr[WI_FILENAME],           wi_filename_help) ;
  help.add_help(warr[WI_HEADERS],            wi_headers_help) ;
  help.add_help(warr[WI_FILE_TYPE_LABEL],    wi_file_type_help) ;
  help.add_help(warr[WI_FILE_TYPE],          wi_file_type_help) ;
  help.add_help(warr[WI_ORGANISER],          wi_org_type_help) ;
  help.add_help(warr[WI_ORGANISER_MENU],     wi_org_type_help) ;
  help.add_help(warr[WI_OUT_LABEL],          wi_output_help) ;
  help.add_help(warr[WI_OUT_TYPE],           wi_output_help) ;
  help.add_help(warr[WI_PAPER_SIZE_LABEL],   wi_paper_size_help) ;
  help.add_help(warr[WI_PAPER_SIZE],         wi_paper_size_help) ;
  help.add_help(warr[WI_SUBJECT_LABEL],      wi_subject_help) ;
  help.add_help(warr[WI_SUBJECT],            wi_subject_help) ;
  help.add_help(warr[WI_COPIES_LABEL],       wi_copies_help) ;
  help.add_help(warr[WI_COPIES],             wi_copies_help) ;
  help.add_help(warr[WI_NO_SIDES_LABEL],     wi_no_sides_help) ;
  help.add_help(warr[WI_NO_SIDES],           wi_no_sides_help) ;
  help.add_help(warr[WI_PRINT],              wi_print_help) ;
  help.add_help(warr[WI_RESET],              wi_reset_help) ;
  help.add_help(warr[WI_SAVE],               wi_props_defs_help) ;
  help.add_help(warr[WI_MENU_BAR],           wi_menu_bar_help) ;
  help.add_help(warr[WI_FILE_MB],            wi_menu_bar_help) ;
  help.add_help(warr[WI_FILE_MENU],          wi_menu_bar_help) ;
  help.add_help(warr[WI_EXIT],               wi_exit_help) ;
  help.add_help(warr[WI_OPTIONS],            wi_menu_bar_help) ;
  help.add_help(warr[WI_OPTIONS_MENU],       wi_menu_bar_help) ;
  help.add_help(warr[WI_HEADERS],            wi_header_frame_help) ;
  help.add_help(warr[WI_INPUT],              wi_input_frame_help) ;
  help.add_help(warr[WI_OUTPUT],             wi_output_frame_help) ;
  help.add_help(warr[WI_PRINTING],           wi_props_frame_help) ;
  help.add_help(warr[WI_SAVE],               wi_props_defs_help) ;
  help.add_help(warr[WI_USERNAME],           wi_un_form_help) ;
  help.add_help(warr[WI_HELP],               wi_help_help) ;
  help.add_help(warr[WI_HELP_MENU],          wi_menu_bar_help) ;
  help.add_help(warr[WI_GENERAL_HELP],       wi_main_frame_help) ;
  help.add_help(warr[WI_HEADERS_HELP],       wi_header_frame_help) ;
  help.add_help(warr[WI_INPUT_HELP],         wi_input_frame_help) ;
  help.add_help(warr[WI_OUTPUT_HELP],        wi_output_frame_help) ;
  help.add_help(warr[WI_PRINTING_HELP],      wi_props_frame_help) ;
  help.add_help(warr[WI_SAVE_HELP],          wi_props_defs_help) ;
  help.add_help(warr[WI_USERNAME_HELP],      wi_un_form_help) ;

  dpy    = tu_disp_get_dpy(NULL) ;
  screen = DefaultScreen(dpy) ;
  root   = RootWindow(dpy, screen) ;
  depth  = DefaultDepth(dpy, screen) ;
  if (DisplayCells(dpy, screen) > 2)
    {
      nicon = x.create_mp_pixmap(u.concat_strs(mptool_xpm_bits)) ;
      picon = x.create_mp_pixmap(u.concat_strs(printing_xpm_bits)) ;
    }
  else
    { 
      nicon = XCreatePixmapFromBitmapData(dpy, RootWindow(dpy, screen),
                      (char *) mptool_xbm_bits,
                      mptool_xbm_width, mptool_xbm_height, 1, 0, 1) ;
      picon = XCreatePixmapFromBitmapData(dpy, RootWindow(dpy, screen),
                      (char *) printing_xbm_bits,
                      printing_xbm_width, printing_xbm_height, 1, 0, 1) ;
    }
  SPRINTF(title, mptool_strs[(int) S_USAGE], PATCHLEVEL) ;
  XtVaSetValues(warr[WI_MAIN_FRAME],
                XmNiconName,   "mptool",
                XmNiconPixmap, nicon,
                XmNtitle,      title,
                0) ;
}


void Base::
do_show_file(void)
{
  if (!fpopup)
    {
      fpopup = tu_file_frame_widget("filename", base.warr[WI_MAIN_FRAME],
                                         &fwarr) ;
      XtRealizeWidget(fpopup) ;
      XSync(dpy, 0) ;
      x.add_delete_callback(fpopup) ;
    }
  x.show_popup(fpopup) ;
}


void Base::
get_main_vals(void)
{
  opt.article   = x.get_bool_value(F_MAIN, WI_FILE_TYPE_NEWS) ;
  opt.print_ps  = x.get_bool_value(F_MAIN, WI_FILE_TYPE_POST) ;
  opt.text_doc  = x.get_bool_value(F_MAIN, WI_FILE_TYPE_TEXT) ;
  opt.gsubject  = x.get_str_value(F_MAIN,  WI_SUBJECT) ;
  opt.copies    = x.get_int_value(F_MAIN,  WI_COPIES) ;
  opt.duplex    = x.get_bool_value(F_MAIN, WI_NO_SIDES_DOUBLE) ;
  opt.landscape = x.get_bool_value(F_MAIN, WI_OUTPUT_LAND) ;

  if (opt.landscape) opt.ptype = PRO_LANDSCAPE ;
  else               opt.ptype = PRO_NORMAL ;

  if (x.get_bool_value(F_MAIN, WI_OUTPUT_PERORG))
    {
      if (x.get_bool_value(F_MAIN, WI_ORG_TYPE_FF))
        opt.ptype = PRO_FILOFAX ;
      if (x.get_bool_value(F_MAIN, WI_ORG_TYPE_FP))
        opt.ptype = PRO_FRANKLIN ;
      if (x.get_bool_value(F_MAIN, WI_ORG_TYPE_PP))
        opt.ptype = PRO_PRO_PLAN ;
      if (x.get_bool_value(F_MAIN, WI_ORG_TYPE_TM))
        opt.ptype = PRO_TIME_MANAGER ;

      if (x.get_bool_value(F_MAIN, WI_ORG_TYPE_TSI))
        opt.ptype = PRO_TS_INTL ;
      if (x.get_bool_value(F_MAIN, WI_ORG_TYPE_TSP))
        opt.ptype = PRO_TS_PARTNER ;
    }

       if (x.get_bool_value(F_MAIN, WI_PAPER_SIZE_A4)) opt.paper_size = P_A4 ;
  else if (x.get_bool_value(F_MAIN, WI_PAPER_SIZE_US)) opt.paper_size = P_US ;
  else                                                 opt.paper_size = P_NONE ;
}


void Base::
set_icon(enum icon_type itype)
{
  Pixmap icon = (itype == I_NORMAL) ? nicon : picon ;

  XtVaSetValues(warr[WI_MAIN_FRAME], XmNiconPixmap, icon, 0) ;
}
