
//  @(#)toolfuns.C 1.9 94/02/03
//
//  Various routines used by the mptool program.
//
//  Copyright (c) Rich Burridge.    All rights reserved.
//
//  Permission is given to distribute these sources, as long as the
//  copyright messages are not removed, and no monies are exchanged.
//
//  No responsibility is taken for any errors or inaccuracies inherent
//  either to the comments or the code of this program, but if
//  reported to me then an attempt will be made to fix them.

#include "mptool.h"
#include <ctype.h>
#include <sys/param.h>
#include <sys/stat.h>

extern "C" {
#include "tu_runtime.h"
}

Base   base ;
Header hdr ;
Help   help ;
Input  input ;
Option opt ;
Output output ;
Print  print ;
User   user ;
Util   u ;
Xutil  x ;

static void exec_print_job  (char *) ;
static void get_main_vals   (void) ;


void
do_print_cmd(char *filenames)
{
  char buf[BUFFERSIZE], m[MAXLINE] ;
  int i ;

  int copies = x.get_int_value(F_MAIN, WI_COPIES) ;

  base.set_icon(I_PRINTING) ;

#ifdef DEBUG
  STRCPY(buf, "echo mp ") ;
#else
  STRCPY(buf, "mp ") ;
#endif // DEBUG

  u.buf_arg(buf, "-printer") ;                        // -printer
  if (opt.printer_name && *opt.printer_name != '\0')
    u.buf_str(buf, opt.printer_name) ;

  if (x.get_bool_value(F_MAIN, WI_FILE_TYPE_NEWS))    // -article
    u.buf_arg(buf, "-article") ;
  if (x.get_bool_value(F_MAIN, WI_FILE_TYPE_POST))    // -postscript
    u.buf_arg(buf, "-postscript") ;
  if (x.get_bool_value(F_MAIN, WI_FILE_TYPE_TEXT))    // -text
    u.buf_arg(buf, "-text") ;

  if (x.get_bool_value(F_MAIN, WI_OUTPUT_PERORG))
    {
      if (x.get_bool_value(F_MAIN, WI_ORG_TYPE_FP))   // -franklin
        u.buf_arg(buf, "-franklin") ;
      if (x.get_bool_value(F_MAIN, WI_ORG_TYPE_FF))   // -filofax
        u.buf_arg(buf, "-filofax") ;
      if (x.get_bool_value(F_MAIN, WI_ORG_TYPE_PP))   // -proplan
        u.buf_arg(buf, "-proplan") ;
      if (x.get_bool_value(F_MAIN, WI_ORG_TYPE_TM))   // -timeman
        u.buf_arg(buf, "-timeman") ;
      if (x.get_bool_value(F_MAIN, WI_ORG_TYPE_TSI))  // -tmintl
        u.buf_arg(buf, "-tsintl") ;
      if (x.get_bool_value(F_MAIN, WI_ORG_TYPE_TSP))  // -tmpartner
        u.buf_arg(buf, "-tspartner") ;
    }

  if (x.get_bool_value(F_MAIN, WI_OUTPUT_LAND))       // -landscape
    u.buf_arg(buf, "-landscape") ;

  if (x.get_bool_value(F_MAIN, WI_PAPER_SIZE_A4))     // -a4
    u.buf_arg(buf, "-a4") ;
  if (x.get_bool_value(F_MAIN, WI_PAPER_SIZE_US))     // -us
    u.buf_arg(buf, "-us") ;

  if (!opt.banner) u.buf_arg(buf, "-nobanner") ;      // -nobanner
  if (opt.folder)  u.buf_arg(buf, "-folder") ;        // -folder
  if (opt.digest)  u.buf_arg(buf, "-digest") ;        // -digest

  if (opt.gsubject && *opt.gsubject != '\0')          // -subject text
    {
      u.buf_arg(buf, "-subject") ;
      u.buf_str(buf, opt.gsubject) ;
    }

  if (opt.llen < 1 || opt.llen > MAXLINELENGTH)       // -linelength #
    {
      SPRINTF(m, mptool_strs[(int) S_INVLINE], LINELENGTH) ;
      x.notice(base.warr[WI_MAIN_FRAME], m) ;
      opt.llen = LINELENGTH ;
      if (output.owarr)
        x.set_int_value(F_OUTPUT, WI_LINE_LENGTH, LINELENGTH) ;
    }
  u.buf_int(buf, "-linelength", opt.llen) ;

  if (opt.plen < 1 || opt.plen > MAXPAGELENGTH)       // -pagelength #
    {
      SPRINTF(m, mptool_strs[(int) S_INVPAGE], PAGELENGTH) ;
      x.notice(base.warr[WI_MAIN_FRAME], m) ;
      opt.plen = PAGELENGTH ;
      if (output.owarr)
        x.set_int_value(F_OUTPUT, WI_PAGE_LENGTH, PAGELENGTH) ;
    }
  u.buf_int(buf, "-pagelength", opt.plen) ;

  if (opt.allhdrs) u.buf_arg(buf, "-allhdrs") ;       // -allhdrs
  else
    {
      for (i = 0; i < opt.remhdr_num; i++)            // -remhdr <hdr>
        {
          u.buf_arg(buf, "-remhdr") ;
          u.buf_str(buf, opt.remhdrs[i]) ;
        }

      for (i = 0; i < opt.addhdr_num; i++)            // -addhdr <hdr>
        {
          u.buf_arg(buf, "-addhdr") ;
          u.buf_str(buf, opt.addhdrs[i]) ;
        }
    }

  if (copies < 1 || copies > MAXCOPIES)             // -copies #
    {
      SPRINTF(m, mptool_strs[(int) S_INVCOPIES], DEFCOPIES) ;
      x.notice(base.warr[WI_MAIN_FRAME], m) ;
      copies = DEFCOPIES ;
      x.set_int_value(F_MAIN, WI_COPIES, 1) ;
    }
  u.buf_int(buf, "-copies", copies) ;

  if (x.get_bool_value(F_MAIN, WI_NO_SIDES_DOUBLE))
    u.buf_arg(buf, "-twosided") ;                     // -twosided
  else
    u.buf_arg(buf, "-onesided") ;                     // -onesided

  if (opt.aliasname && *opt.aliasname != '\0')
    {
      u.buf_arg(buf, "-alias") ;
      u.buf_str(buf, opt.aliasname) ;
    }

  if (opt.proname && *opt.proname != '\0')          // -prologue name
    {
      u.buf_arg(buf, "-prologue") ;
      u.buf_str(buf, opt.proname) ;
    }

  u.buf_int(buf, "-columns", opt.numcols) ;           // -columns #
  u.buf_int(buf, "-tab",     opt.tabsize) ;           // -tab #
  u.buf_int(buf, "-chars",   opt.namelength) ;        // -chars #
  u.buf_int(buf, "-words",   opt.namefields) ;        // -words #
  if (opt.wrapwords)  u.buf_arg(buf, "-wrap") ;       // -wrap
  if (opt.flip)       u.buf_arg(buf, "-flip") ;       // -flip
  if (opt.forcepage)  u.buf_arg(buf, "-forcepage") ;  // -forcepage
  if (opt.elm_if)     u.buf_arg(buf, "-elm") ;        // -elm
  if (opt.number)     u.buf_arg(buf, "-number") ;     // -number
  if (opt.print_orig) u.buf_arg(buf, "-from") ;       // -from
  if (opt.content)    u.buf_arg(buf, "-content") ;    // -content
  if (opt.modtime)    u.buf_arg(buf, "-modtime") ;    // -modtime
  if (opt.bsdprint)   u.buf_arg(buf, "-bsdprint") ;   // -bsdprint
  if (opt.tumble)     u.buf_arg(buf, "-tumble") ;     // -tumble

  u.buf_int(buf, "-bottom", opt.bmargin) ;            // -bottom #
  u.buf_int(buf, "-left",   opt.lmargin) ;            // -left   #
  u.buf_int(buf, "-right",  opt.rmargin) ;            // -right  #
  u.buf_int(buf, "-top",    opt.tmargin) ;            // -top    #

  if (filenames == NULL || *filenames == '\0')
    {
      x.notice(base.warr[WI_MAIN_FRAME], mptool_strs[(int) S_NOFILES]) ;
      return ;
    }
  u.buf_arg(buf, filenames) ;
  exec_print_job(buf) ;
  base.set_icon(I_NORMAL) ;
}


void
do_startup(void)
{
  char **argv, files[BUFFERSIZE], title[MAXLINE] ;
  int argc, i ;

  argc = tu_get_cmd_argc() ;
  argv = (char **) calloc(argc, sizeof(char *)) ;
  for (i = 0; i < argc; i++)
    argv[i] = tu_get_cmd_argv(i) ;

  base   = Base() ;
  hdr    = Header() ;
  help   = Help() ;
  input  = Input() ;
  opt    = Option() ;
  opt.get_appname(argc, argv) ;
  output = Output() ;
  print  = Print() ;
  user   = User() ;
  u      = Util() ;
  x      = Xutil() ;

  hdr.init_headers() ;           // Initialise standard headers.
  init_text() ;                  // Internationalise text strings.
  opt.read_resources() ;         // Read resources from merged database.
  opt.get_options(argc, argv) ;  // Read and process command line options
  base.do_show_base() ;          // Create base window.

  files[0] = '\0' ;                     // Prime the list of files to print.
  for (i = 0; i < opt.numnames; i++)
    {
      STRCAT(files, opt.namelist[i]) ;
      STRCAT(files, " ") ;
    }
  files[strlen(files)-1] = '\0' ;
  u.read_str(&base.filenames, files) ;

  base.do_reset_options() ;
  XtRealizeWidget(base.warr[WI_MAIN_FRAME]) ;
  XmProcessTraversal(base.warr[WI_FILENAME], XmTRAVERSE_CURRENT) ;
  XtMapWidget(base.warr[WI_MAIN_FRAME]) ;
}


static void
exec_print_job(char *buf)
{
  int pid ;

  if ((pid = (int) fork()) < 0)
    {
      x.notice(base.warr[WI_MAIN_FRAME], mptool_strs[(int) S_START]) ;
      return ;
    }

  if (pid == 0)           // Child process.
    {
      (void) execl("/bin/sh", "sh", "-c", buf, (char *) 0) ;
      perror(mptool_strs[(int) S_EXECUTE]) ;
      _exit(1) ;
    }
}


Widget
get_widget(enum frame_type ftype, int index)
{
       if (ftype == F_MAIN   && base.warr)    return(base.warr[index]) ;
  else if (ftype == F_HDRS   && hdr.hwarr)    return(hdr.hwarr[index]) ;
  else if (ftype == F_INPUT  && input.iwarr)  return(input.iwarr[index]) ;
  else if (ftype == F_OUTPUT && output.owarr) return(output.owarr[index]) ;
  else if (ftype == F_PRINT  && print.pwarr)  return(print.pwarr[index]) ;
  else if (ftype == F_USER   && user.uwarr)   return(user.uwarr[index]) ;
  else return(NULL) ;                         // Shouldn't happen.
}
